@@ -34,6 +34,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
+#include <X11/Xatom.h>
 
 #include "gsd-enums.h"
 #include "gsd-input-helper.h"
@@ -41,9 +42,6 @@
 #include "gsd-wacom-manager.h"
 #include "gsd-wacom-device.h"
 
-/* Maximum number of buttons map-able. */
-#define WACOM_MAX_BUTTONS 32
-
 #define GSD_WACOM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_MANAGER, GsdWacomManagerPrivate))
 
 #define KEY_ROTATION            "rotation"
@@ -52,6 +50,7 @@
 #define KEY_IS_ABSOLUTE         "is-absolute"
 #define KEY_AREA                "area"
 #define KEY_PAD_BUTTON_MAPPING  "pad-buttonmapping"
+#define KEY_DISPLAY             "display"
 
 /* Stylus and Eraser settings */
 #define KEY_BUTTON_MAPPING      "buttonmapping"
@@ -151,6 +150,7 @@ set_rotation (GsdWacomDevice *device,
                 .name = "Wacom Rotation",
                 .nitems = 1,
                 .format = 8,
+                .type   = XA_INTEGER,
                 .data.c = &rot,
         };
 
@@ -164,11 +164,13 @@ set_pressurecurve (GsdWacomDevice *device,
         PropertyHelper property = {
                 .name = "Wacom Pressurecurve",
                 .nitems = 4,
+                .type   = XA_INTEGER,
                 .format = 32,
         };
         gsize nvalues;
 
         property.data.i = g_variant_get_fixed_array (value, &nvalues, sizeof (gint32));
+        g_variant_unref (value);
 
         if (nvalues != 4) {
                 g_error ("Pressurecurve requires 4 values.");
@@ -188,11 +190,13 @@ set_area (GsdWacomDevice  *device,
         PropertyHelper property = {
                 .name = "Wacom Tablet Area",
                 .nitems = 4,
+                .type   = XA_INTEGER,
                 .format = 32,
         };
         gsize nvalues;
 
         property.data.i = g_variant_get_fixed_array (value, &nvalues, sizeof (gint32));
+        g_variant_unref (value);
 
         if (nvalues != 4) {
                 g_error ("Area configuration requires 4 values.");
@@ -202,6 +206,27 @@ set_area (GsdWacomDevice  *device,
         wacom_set_property (device, &property);
 }
 
+static void
+set_display (GsdWacomDevice  *device,
+             GVariant        *value)
+{
+        float matrix[NUM_ELEMS_MATRIX];
+        PropertyHelper property = {
+                .name   = "Coordinate Transformation Matrix",
+                .nitems = NUM_ELEMS_MATRIX,
+                .format = 32,
+                .type   = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "FLOAT", True),
+        };
+
+        gsd_wacom_device_get_display_matrix (device, matrix);
+
+        property.data.i = (gint*)(&matrix);
+        g_debug ("Applying matrix to device...");
+        wacom_set_property (device, &property);
+
+        g_variant_unref (value);
+}
+
 static void
 set_absolute (GsdWacomDevice  *device,
               gint             is_absolute)
@@ -224,14 +249,16 @@ set_device_buttonmap (GsdWacomDevice *device,
 	XDevice *xdev;
 	gsize nmap;
 	const gint *intmap;
-	unsigned char map[WACOM_MAX_BUTTONS] = {};
+	unsigned char *map;
 	int i, j, rc;
 
 	xdev = open_device (device);
 
 	intmap = g_variant_get_fixed_array (value, &nmap, sizeof (gint32));
+	map = g_new0 (unsigned char, nmap);
 	for (i = 0; i < nmap && i < sizeof (map); i++)
 		map[i] = intmap[i];
+        g_variant_unref (value);
 
 	gdk_error_trap_push ();
 
@@ -247,6 +274,8 @@ set_device_buttonmap (GsdWacomDevice *device,
 	if (gdk_error_trap_pop () || rc != Success)
 		g_warning ("Error in setting button mapping for \"%s\"", gsd_wacom_device_get_tool_name (device));
 
+	g_free (map);
+
 	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
 }
 
@@ -259,6 +288,7 @@ set_touch (GsdWacomDevice *device,
                 .name = "Wacom Enable Touch",
                 .nitems = 1,
                 .format = 8,
+                .type   = XA_INTEGER,
                 .data.c = &data,
         };
 
@@ -278,6 +308,7 @@ set_tpcbutton (GsdWacomDevice *device,
                 .name = "Wacom Hover Click",
                 .nitems = 1,
                 .format = 8,
+                .type   = XA_INTEGER,
                 .data.c = &data,
         };
 
@@ -292,6 +323,7 @@ set_pressurethreshold (GsdWacomDevice *device,
                 .name = "Wacom Pressure Threshold",
                 .nitems = 1,
                 .format = 32,
+                .type   = XA_INTEGER,
                 .data.i = &threshold,
         };
 
@@ -348,7 +380,6 @@ set_wacom_settings (GsdWacomManager *manager,
 
 		variant = g_variant_new_array (G_VARIANT_TYPE_INT32, values, G_N_ELEMENTS (values));
 		set_area (device, variant);
-		g_variant_unref (variant);
 		return;
 	}
 
@@ -362,6 +393,7 @@ set_wacom_settings (GsdWacomManager *manager,
 
 	set_absolute (device, g_settings_get_boolean (settings, KEY_IS_ABSOLUTE));
 	set_area (device, g_settings_get_value (settings, KEY_AREA));
+	set_display (device, g_settings_get_value (settings, KEY_DISPLAY));
 
         /* only pen and eraser have pressure threshold and curve settings */
         if (type == WACOM_TYPE_STYLUS ||
@@ -396,6 +428,10 @@ wacom_settings_changed (GSettings      *settings,
 	} else if (g_str_equal (key, KEY_PAD_BUTTON_MAPPING)) {
 		if (type == WACOM_TYPE_PAD)
 			set_device_buttonmap (device, g_settings_get_value (settings, key));
+	} else if (g_str_equal (key, KEY_DISPLAY)) {
+		if (type != WACOM_TYPE_CURSOR &&
+		    type != WACOM_TYPE_PAD)
+			set_display (device, g_settings_get_value (settings, key));
 	} else {
 		g_warning ("Unhandled tablet-wide setting '%s' changed", key);
 	}
