@@ -27,6 +27,7 @@
 
 static gboolean fake_devices = FALSE;
 static gboolean monitor_styli = FALSE;
+static gboolean option_debug = FALSE;
 
 static char *
 get_loc (GSettings *settings)
@@ -46,29 +47,6 @@ get_loc (GSettings *settings)
 	return ret;
 }
 
-static void
-last_stylus_changed (GsdWacomDevice  *device,
-		     GParamSpec      *pspec,
-		     gpointer         user_data)
-{
-	GsdWacomStylus *stylus;
-	char *loc;
-
-	g_object_get (device, "last-stylus", &stylus, NULL);
-
-	g_print ("Stylus changed for device '%s'\n",
-		 gsd_wacom_device_get_name (device));
-
-	g_print ("\t*** Stylus: '%s'\n",
-		 gsd_wacom_stylus_get_name (stylus));
-
-	loc = get_loc (gsd_wacom_stylus_get_settings (stylus));
-	g_print ("\t\tSettings: %s\n", loc);
-	g_free (loc);
-
-	g_print ("\t\tIcon name: %s\n", gsd_wacom_stylus_get_icon_name (stylus));
-}
-
 static const char *
 stylus_type_to_string (GsdWacomStylusType type)
 {
@@ -87,14 +65,110 @@ stylus_type_to_string (GsdWacomStylusType type)
 		return "Marker";
 	case WACOM_STYLUS_TYPE_STROKE:
 		return "Stroke";
+	case WACOM_STYLUS_TYPE_PUCK:
+		return "Puck";
 	default:
 		g_assert_not_reached ();
 	}
 	return NULL;
 }
 
+static const char *
+button_type_to_string (GsdWacomTabletButtonType type)
+{
+	switch (type) {
+	case WACOM_TABLET_BUTTON_TYPE_NORMAL:
+		return "normal";
+	case WACOM_TABLET_BUTTON_TYPE_ELEVATOR:
+		return "elevator";
+	case WACOM_TABLET_BUTTON_TYPE_HARDCODED:
+		return "hard-coded";
+	default:
+		g_assert_not_reached ();
+	}
+}
+
 #define BOOL_AS_STR(x) (x ? "yes" : "no")
 
+static void
+print_stylus (GsdWacomStylus *stylus,
+	      gboolean        is_current)
+{
+	GsdWacomDevice *device;
+	char *loc;
+
+	device = gsd_wacom_stylus_get_device (stylus);
+
+	g_print ("\t%sStylus: '%s' (Type: %s, ID: 0x%x)\n",
+		 is_current ? "*** " : "",
+		 gsd_wacom_stylus_get_name (stylus),
+		 stylus_type_to_string (gsd_wacom_stylus_get_stylus_type (stylus)),
+		 gsd_wacom_stylus_get_id (stylus));
+
+	loc = get_loc (gsd_wacom_stylus_get_settings (stylus));
+	g_print ("\t\tSettings: %s\n", loc);
+	g_free (loc);
+
+	g_print ("\t\tIcon name: %s\n", gsd_wacom_stylus_get_icon_name (stylus));
+
+	if (gsd_wacom_device_get_device_type (device) == WACOM_TYPE_STYLUS) {
+		int num_buttons;
+		char *buttons;
+
+		g_print ("\t\tHas Eraser: %s\n", BOOL_AS_STR(gsd_wacom_stylus_get_has_eraser (stylus)));
+
+		num_buttons = gsd_wacom_stylus_get_num_buttons (stylus);
+		if (num_buttons < 0)
+			num_buttons = 2;
+		if (num_buttons > 0)
+			buttons = g_strdup_printf ("%d buttons", num_buttons);
+		else
+			buttons = g_strdup ("no button");
+		g_print ("\t\tButtons: %s\n", buttons);
+		g_free (buttons);
+	}
+}
+
+static void
+print_buttons (GsdWacomDevice *device)
+{
+	GList *buttons, *l;
+
+	buttons = gsd_wacom_device_get_buttons (device);
+	if (buttons == NULL)
+		return;
+
+	g_print ("\tButtons:\n");
+	for (l = buttons; l != NULL; l = l->next) {
+		GsdWacomTabletButton *button = l->data;
+
+		g_print ("\t\t%s ('%s', type: %s",
+			 button->name,
+			 button->id,
+			 button_type_to_string (button->type));
+		if (button->group_id > 0)
+			g_print (", group: %d)\n", button->group_id);
+		else
+			g_print (")\n");
+	}
+	g_list_free (buttons);
+}
+
+static void
+last_stylus_changed (GsdWacomDevice  *device,
+		     GParamSpec      *pspec,
+		     gpointer         user_data)
+{
+	GsdWacomStylus *stylus;
+
+	g_object_get (device, "last-stylus", &stylus, NULL);
+
+	g_print ("Stylus changed for device '%s'\n",
+		 gsd_wacom_device_get_name (device));
+
+	print_stylus (stylus, TRUE);
+}
+
 static void
 list_devices (GList *devices)
 {
@@ -133,37 +207,13 @@ list_devices (GList *devices)
 				GsdWacomStylus *stylus;
 
 				stylus = j->data;
-				g_print ("\t%sStylus: '%s' (Type: %s)\n",
-					 current_stylus == stylus ? "*** " : "",
-					 gsd_wacom_stylus_get_name (stylus),
-					 stylus_type_to_string (gsd_wacom_stylus_get_stylus_type (stylus)));
-
-				loc = get_loc (gsd_wacom_stylus_get_settings (stylus));
-				g_print ("\t\tSettings: %s\n", loc);
-				g_free (loc);
-
-				g_print ("\t\tIcon name: %s\n", gsd_wacom_stylus_get_icon_name (stylus));
-
-				if (gsd_wacom_device_get_device_type (device) == WACOM_TYPE_STYLUS) {
-					int num_buttons;
-					char *buttons;
-
-					g_print ("\t\tHas Eraser: %s\n", BOOL_AS_STR(gsd_wacom_stylus_get_has_eraser (stylus)));
-
-					num_buttons = gsd_wacom_stylus_get_num_buttons (stylus);
-					if (num_buttons < 0)
-						num_buttons = 2;
-					if (num_buttons > 0)
-						buttons = g_strdup_printf ("%d buttons", num_buttons);
-					else
-						buttons = g_strdup ("no button");
-					g_print ("\t\tButtons: %s\n", buttons);
-					g_free (buttons);
-				}
+				print_stylus (stylus, current_stylus == stylus);
 			}
 			g_list_free (styli);
 		}
 
+		print_buttons (device);
+
 		if (monitor_styli == FALSE)
 			g_object_unref (device);
 	}
@@ -221,6 +271,7 @@ int main (int argc, char **argv)
 	const GOptionEntry entries[] = {
 		{ "fake", 'f', 0, G_OPTION_ARG_NONE, &fake_devices, "Output fake devices", NULL },
 		{ "monitor", 'm', 0, G_OPTION_ARG_NONE, &monitor_styli, "Monitor changing styli", NULL },
+		{ "debug", 'd', 0, G_OPTION_ARG_NONE, &option_debug, "Debug output", NULL },
 		{ NULL }
 	};
 
@@ -234,6 +285,9 @@ int main (int argc, char **argv)
 		return 1;
 	}
 
+	if (option_debug)
+		g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
+
 	if (fake_devices == FALSE)
 		list_actual_devices ();
 	else
