@@ -858,6 +858,7 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                 val = g_settings_get_value (settings, translations[i].gsettings_key);
 
                 process_value (manager, &translations[i], val);
+                g_variant_unref (val);
         }
 
         list = g_hash_table_get_values (manager->priv->settings);
@@ -929,7 +930,8 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
 
         stop_fontconfig_monitor (manager);
 
-        g_bus_unwatch_name (manager->priv->shell_name_watch_id);
+        if (manager->priv->shell_name_watch_id > 0)
+                g_bus_unwatch_name (manager->priv->shell_name_watch_id);
 
         if (p->settings != NULL) {
                 g_hash_table_destroy (p->settings);
